##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Putfactory registration tool interface.

$Id: ContentTypeRegistry.py 38590 2005-09-24 15:24:32Z yuppie $
"""

# BBB: module will be removed in CMF 2.2
#      zope2 interfaces created on runtime:
#      - ContentTypeRegistryPredicate
#      - ContentTypeRegistry
